const { oneLine } = require('common-tags');
const Command = require('../base');

module.exports = class DisableCommandCommand extends Command {
	constructor(client) {
		super(client, {
			name: 'disable',
			aliases: ['disable-command', 'cmd-off', 'command-off'],
			group: 'commands',
			memberName: 'disable',
			description: 'Desactiva un comando o grupo de comandos.',
			details: oneLine`
				El argumento debe ser el nombre/ID (parcial o completo) de un comando o un grupo de comandos.
				Solo los usuarios en la owner list pueden usar este comando.
			`,
			examples: ['disable util', 'disable Utility', 'disable prefix'],
			guarded: true,

			args: [
				{
					key: 'cmdOrGrp',
					label: 'command/group',
					prompt: '¿Qué comando o grupo de comandos te gustaría desactivar?',
					type: 'group|command'
				}
			]
		});
	}

	hasPermission(msg) {
		if(!msg.guild) return this.client.isOwner(msg.author);
		return msg.member.hasPermission('ADMINISTRATOR') || this.client.isOwner(msg.author);
	}

	run(msg, args) {
		if(!args.cmdOrGrp.isEnabledIn(msg.guild, true)) {
			return msg.reply(
				`\`${args.cmdOrGrp.name}\` ${args.cmdOrGrp.group ? 'command' : 'group'} ya está desactivado.`
			);
		}
		if(args.cmdOrGrp.guarded) {
			return msg.reply(
				`No puedes desactivar \`${args.cmdOrGrp.name}\` ${args.cmdOrGrp.group ? 'command' : 'group'}.`
			);
		}
		args.cmdOrGrp.setEnabledIn(msg.guild, false);
		return msg.reply(`\`${args.cmdOrGrp.name}\` ${args.cmdOrGrp.group ? 'command' : 'group'} desactivado.`);
	}
};
