const { oneLine } = require('common-tags');
const Command = require('../base');

module.exports = class EnableCommandCommand extends Command {
	constructor(client) {
		super(client, {
			name: 'enable',
			aliases: ['enable-command', 'cmd-on', 'command-on'],
			group: 'commands',
			memberName: 'enable',
			description: 'Activa un comando o un grupo de comandos.',
			details: oneLine`
				El argumento debe ser el nombre/ID (parcial o completo) de un comando o grupo de comandos.
				Only administrators may use this command.
			`,
			examples: ['enable util', 'enable Utility', 'enable prefix'],
			guarded: true,

			args: [
				{
					key: 'cmdOrGrp',
					label: 'command/group',
					prompt: '¿Qué comando o grupo de comandos quieres activar?',
					type: 'group|command'
				}
			]
		});
	}

	hasPermission(msg) {
		if(!msg.guild) return this.client.isOwner(msg.author);
		return msg.member.hasPermission('ADMINISTRATOR') || this.client.isOwner(msg.author);
	}

	run(msg, args) {
		const group = args.cmdOrGrp.group;
		if(args.cmdOrGrp.isEnabledIn(msg.guild, true)) {
			return msg.reply(
				`\`${args.cmdOrGrp.name}\` ${args.cmdOrGrp.group ? 'command' : 'group'} ya está activado${
					group && !group.enabled ? `, pero el grupo \`${group.name}\` está desactivado, así que todavía no puede ser usado.` : ''
				}.`
			);
		}
		args.cmdOrGrp.setEnabledIn(msg.guild, true);
		return msg.reply(
			`\`${args.cmdOrGrp.name}\` ${group ? 'command' : 'group'}${
				group && !group.enabled ? ` activado, pero el grupo \`${group.name}\` está desactivado, así que todavía no puede ser usado.` : ''
			}.`
		);
	}
};
